/*
 * Copyright 2003, 2004 Porchdog Software. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY PORCHDOG SOFTWARE ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Porchdog Software.
 */

#include <salt/log.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define SW_LOG_BUFFER_SIZE		256

static sw_bool g_enable_debug = SW_FALSE;


static sw_const_string
log_type_name(
			int level);
			
			
static sw_const_string
base_file_name(
			sw_const_string file_name);
			

void HOWL_DLL
_sw_log_header(
				sw_const_string	file,
				int 					line)
{
   fprintf(stderr, "%s:%d: ", base_file_name(file), line);
}


void
_sw_log(	
			int					level,
			sw_const_string	function, 
			sw_const_string	format, 
			...)
{
	va_list	arglist;
	char		buf[SW_LOG_BUFFER_SIZE];
	
	if (level == SW_LOG_INFO)
	{
		fprintf(stderr, "%s: ", function);
	}
	else
	{
		fprintf(stderr, "%s %s: ", log_type_name(level), function);
	}
	va_start(arglist, format);
	vsprintf(buf, format, arglist );
	va_end(arglist);
	fprintf(stderr, buf);
	
#if defined(WIN32)
	OutputDebugString(buf);
#endif

	if (level == SW_LOG_FATAL_ERROR)
	{
		exit(-1);
	}
}


void HOWL_DLL
sw_debug_enable(
				sw_bool val)
{
	g_enable_debug = val;
}


void
_sw_debug_header(
				sw_const_string	file,
				int 					line)
{
	if (g_enable_debug)
	{
		fprintf(stderr, "%s:%d: ", base_file_name(file), line);
	}
}


void
_sw_debug(	
				sw_const_string	format,
				...)
{
	if (g_enable_debug)
	{
		va_list	arglist;
		char		buf[SW_LOG_BUFFER_SIZE];
		
		va_start( arglist, format );
		vsprintf(buf, format, arglist );
		va_end( arglist );
		fprintf(stderr, buf);
#if defined(WIN32)
		OutputDebugString(buf);
#endif
	}
}


static sw_const_string
log_type_name(
			int level)
{
	static sw_string names[] =
	{
		"",
		"warning:",
		"error:",
		"fatal error:",
		"unknown error:"
	};
		
	if (level < 4)
	{
		return names[level];
	}
	else
	{
		return names[4];
	}
}


static sw_const_string
base_file_name(
			sw_const_string file_name)
{
	int i;
		
	for (i = strlen(file_name); i >= 0; i--)
	{
		if ((file_name[i] == '/') || (file_name[i] == '\\'))
		{
			return (file_name + i + 1);
		}
	}
	
	return file_name;
}
