/*
 * Copyright 2003, 2004 Porchdog Software. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY PORCHDOG SOFTWARE ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Porchdog Software.
 */

#include "object_i.h"
#include "channel_i.h"
#include <salt/vstdlib.h>
#include <salt/vstring.h>
#include <salt/verrno.h>
#include <salt/assert.h>


sw_result
sw_corby_object_init(
							sw_corby_object	*	self)
{
	SW_TRY(sw_malloc((*self), sw_corby_object, sizeof(struct _sw_corby_object)));

	(*self)->m_iors		=	NULL;
	(*self)->m_channel	=	NULL;
	(*self)->m_channel	=	NULL;
	(*self)->m_options	=	NULL;
	(*self)->m_bufsize	=	4192;
	(*self)->m_profile	=	NULL;

	return SW_OKAY;
}


sw_result HOWL_DLL
sw_corby_object_init_from_url(
							sw_corby_object		*	self,
							struct _sw_corby_orb	*	orb,
							const sw_string			url,
							sw_socket_options			options,
							sw_ulong						bufsize)
{
   sw_string			port_spec;
   sw_string			host;
   sw_string			oid;
   char					buf[256];
   sw_string			protocol_spec;
   sw_string			oid_spec;
   sw_string			tmp;
	sw_result			result;
	sw_corby_ior		ior;
	sw_corby_profile	profile;

	/*
	 * initialize
	 */
	ior		= NULL;
	profile	= NULL;
	
	
	/*
	 * try and allocate 
	 */
	SW_TRY(sw_malloc((*self), sw_corby_object, sizeof(struct _sw_corby_object)));
	
	(*self)->m_orb = orb;
	
	SW_TRY(sw_corby_ior_init(&ior));
	SW_TRY(sw_corby_profile_init(&profile));

	if (url == NULL)
	{
		result = SW_E_CORBY_BAD_URL;
		goto exit;
	}
   
   /*
      copy to temporary storage
   */
   sw_strcpy(buf, url);
   
	/*
		find protocol
	*/
	tmp = sw_strchr(buf, ':');
   if (!tmp)
	{
		result = SW_E_CORBY_BAD_URL;
		goto exit;
	}

   *tmp				=	'\0';
   protocol_spec	=	buf;
   if (sw_strcmp(protocol_spec, "iiop") == 0)
   {
      profile->m_tag = SW_TAG_INTERNET_IOP;
   }
   else if (sw_strcmp(protocol_spec, "uiop") == 0)
   {
      profile->m_tag = SW_TAG_UIOP;
   }
   else if (sw_strcmp(protocol_spec, "miop") == 0)
   {
		profile->m_tag = SW_TAG_MIOP;
   }
   else
   {
		result = SW_E_CORBY_BAD_URL;
		goto exit;
   }
   
   /*
	 * make sure we have "//"
	 */
   if ((*++tmp != '/') || (*++tmp != '/'))
   {
		result = SW_E_CORBY_BAD_URL;
		goto exit;
   }
   
	/*
	 * increment past the second slash
	 */
   ++tmp;
   
	/*
	 * check to see if we have a port spec
	 */
	if (port_spec = sw_strchr(tmp, ':'))
	{
		*port_spec	=	'\0';
		host			=	tmp;
		tmp			=  ++port_spec;
      
      oid_spec = sw_strchr(tmp, '/');
      if (!oid_spec)
      {
			result = SW_E_CORBY_BAD_URL;
			goto exit;
      }
      
		*oid_spec	=	'\0';
		port_spec	=	tmp;
		oid			=	++oid_spec;
   }
   else
   {
      port_spec	=	NULL;
      oid_spec		=	sw_strchr(tmp, '/');
      if (!oid_spec)
      {
			result = SW_E_CORBY_BAD_URL;
			goto exit;
      }
      *oid_spec	=	'\0';
      host			=	tmp;
      oid			=	++oid_spec;
   }
   
	SW_TRY(sw_ipv4_address_init_from_name(&profile->m_address, host));
	profile->m_port		=	atoi(port_spec);
	profile->m_oid_len	=	sw_strlen(oid);
	SW_TRY(sw_malloc(profile->m_oid, sw_string, profile->m_oid_len));
	sw_memcpy(profile->m_oid, oid, profile->m_oid_len);
	profile->m_txen		=	NULL;
	ior->m_txen				=	NULL;
	ior->m_num_profiles	=	1;
	ior->m_profiles		=	profile;
	(*self)->m_profile	=	NULL;
	(*self)->m_channel	=	NULL;
	(*self)->m_options	=	options;
	(*self)->m_iors		=	ior;
	(*self)->m_bufsize	=	bufsize;
	
	result = SW_OKAY;

exit:

	if (result != SW_OKAY)
	{
		sw_corby_object_fina((*self));
	}
	
	return result;
}


sw_result HOWL_DLL
sw_corby_object_fina(
							sw_corby_object	self)
{
	if (self != NULL)
	{
		sw_corby_channel_fina(self->m_channel);
		sw_corby_ior_fina(self->m_iors);
		sw_free(self);
	}

	return SW_OKAY;
}


sw_result HOWL_DLL
sw_corby_object_start_request(
							sw_corby_object 		self,
							const sw_string		op,
							sw_ulong					op_len,
							sw_bool					reply_expected,
							sw_corby_buffer	*	buffer)
{
	sw_result result;

	sw_assert(self);
	sw_assert(self->m_iors);
	sw_assert(self->m_iors->m_profiles);

	/*
	 * if we don't have a channel yet then let's create one
	 */
	if (self->m_channel == NULL)
	{
		sw_assert(self->m_profile == NULL);
		self->m_profile = self->m_iors->m_profiles;
		SW_TRY_GOTO(sw_corby_channel_init_with_profile(&self->m_channel, self->m_orb, self->m_profile, self->m_options, self->m_bufsize));
	}

	sw_assert(self->m_profile);
	sw_assert(self->m_channel);

	result = sw_corby_channel_start_request(self->m_channel, self->m_profile, buffer, op, op_len, reply_expected);

exit:

	if (result != SW_OKAY)
	{
		sw_corby_channel_fina(self->m_channel);
		self->m_profile	=	NULL;
		self->m_channel	=	NULL;
	}

	return result;
}


sw_result HOWL_DLL
sw_corby_object_send(
							sw_corby_object					self,
							sw_corby_buffer					buffer,
							sw_corby_buffer_observer		observer,
							sw_corby_buffer_written_func	func,
							sw_opaque							extra)
{
	sw_result result;

	sw_assert(self);
	sw_assert(self->m_iors);
	sw_assert(self->m_iors->m_profiles);
	sw_assert(self->m_channel);

	if ((result = sw_corby_channel_send(self->m_channel, buffer, observer, func, extra)) != SW_OKAY)
	{
		if (result != SW_E_INPROGRESS)
		{
			sw_corby_channel_fina(self->m_channel);
			self->m_profile	=	NULL;
			self->m_channel	=	NULL;
		}

		return result;
	}

	return result;
}


sw_result HOWL_DLL
sw_corby_object_recv(
							sw_corby_object		self,
							sw_corby_message	*	message,
                     sw_corby_buffer	*	buffer,
                     sw_octet				*	endian,
                     sw_bool					block)
{
	sw_result result;

	sw_assert(self);
	sw_assert(self->m_iors);
	sw_assert(self->m_iors->m_profiles);
	sw_assert(self->m_channel);

	if ((result = sw_corby_channel_recv(self->m_channel, NULL, message, NULL, NULL, NULL, buffer, endian, block)) != SW_OKAY)
	{
		sw_corby_channel_fina(self->m_channel);
		self->m_profile	=	NULL;
		self->m_channel	=	NULL;
	}

	return result;
}


sw_result HOWL_DLL
sw_corby_object_channel(
							sw_corby_object		self,
							sw_corby_channel	*	channel)
{
	if (self->m_channel == NULL)
	{
		if (self->m_profile == NULL)
		{
			self->m_profile = self->m_iors->m_profiles;
		}

		SW_TRY(sw_corby_channel_init_with_profile(&self->m_channel, self->m_orb, self->m_profile, self->m_options, self->m_bufsize));
	}

	sw_assert(self->m_channel != NULL);
	*channel = self->m_channel;
	return SW_OKAY;
}


sw_result HOWL_DLL
sw_corby_object_set_channel(
							sw_corby_object		self,
							sw_corby_channel		channel)
{
	sw_assert(self->m_profile == NULL);

	SW_TRY(sw_corby_channel_retain(channel));

	self->m_profile = self->m_iors->m_profiles;
	self->m_channel = channel;
	
	return SW_OKAY;
}
