/*
 * Copyright 2003, 2004 Porchdog Software. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY PORCHDOG SOFTWARE ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Porchdog Software.
 */

#include "socket_i.h"
#include "salt_i.h"
#include "tlist.h"
#include <salt/address.h>
#include <salt/vstdlib.h>
#include <salt/vstring.h>
#include <salt/verrno.h>
#include <salt/assert.h>
#include <salt/log.h>
#if !defined(_WIN32_WCE)
#	include <errno.h>
#endif
#include <stdio.h>

#if defined(WIN32)

#	include <ws2tcpip.h>

#endif

#if defined(__sun)

#	include <sys/filio.h>

#endif


/*
   Private Declarations
*/
static sw_result
sw_socket_init(
            sw_socket					self,
            sw_bool						connected,
            sw_socket_connect_func	connect_func,
            sw_socket_send_func		send_func,
				sw_socket_sendto_func	sendto_func,
            sw_socket_recv_func		recv_func,
				sw_socket_recvfrom_func	recvfrom_func,
            sw_socket_close_func		close_func);

               
static sw_result
sw_socket_tcp_connect(
            sw_socket			self,
            sw_ipv4_address	address,
            sw_port				port);
               
               
static sw_result
sw_socket_tcp_send(
            sw_socket		self,
            sw_octets		buffer,
            sw_len			len,
				sw_ulong		*	bytesWritten);


static sw_result
sw_socket_tcp_sendto(
				sw_socket			self,
				sw_octets			buffer,
				sw_len				len,
				sw_ulong			*	bytesWritten,
				sw_ipv4_address	to,
				sw_port				to_port);
				

static sw_result
sw_socket_tcp_recv(
            sw_socket		self,
            sw_octets		buffer,
            sw_len			max,
            sw_len		*	len);


static sw_result
sw_socket_tcp_recvfrom(
            sw_socket				self,
            sw_octets				buffer,
            sw_len					max,
            sw_len				*	len,
				sw_ipv4_address	*	from,
				sw_port				*	from_port,
				sw_ipv4_address	*	dest,
				sw_ulong				*	interface_index);


static sw_result
sw_socket_tcp_close(
            sw_socket		socket);
               
               
static sw_result
sw_socket_udp_connect(
            sw_socket			self,
            sw_ipv4_address	address,
            sw_port			port);


static sw_result
sw_socket_udp_send(
            sw_socket		self,
            sw_octets		buffer,
            sw_len			len,
				sw_ulong		*	bytesWritten);


static sw_result
sw_socket_udp_sendto(
				sw_socket			self,
				sw_octets			buffer,
				sw_len				len,
				sw_ulong		*		bytesWritten,
				sw_ipv4_address	to,
				sw_port				to_port);
				
				
static sw_result
sw_socket_udp_really_sendto(
				sw_socket				self,
				sw_octets				buffer,
				sw_len					len,
				sw_ulong				*	bytesWritten,
				struct sockaddr	*	to,
				sw_optlen				to_len);


static sw_result
sw_socket_udp_recv(
            sw_socket		self,
            sw_octets		buffer,
            sw_len			max,
            sw_len		*	len);


static sw_result
sw_socket_udp_recvfrom(
				sw_socket				self,
				sw_octets				buffer,
				sw_len					max,
				sw_len				*	len,
				sw_ipv4_address	*	from,
				sw_port				*	from_port,
				sw_ipv4_address	*	dest,
				sw_ulong				*	interface_index);
				

static sw_result
sw_socket_udp_really_recvfrom(
				sw_socket				self,
				sw_octets				buffer,
				sw_len					max,
				sw_len				*	len,
				struct sockaddr	*	from,
				sw_optlen				from_len,
				struct in_addr		*	dest,
				sw_ulong				*	interface_index);


static sw_result
sw_socket_udp_close(
            sw_socket	socket);


/*
   IMPLEMENTATION
*/
sw_result
sw_tcp_socket_super_init(
               sw_socket self)
{
	SW_TRY(sw_socket_init(self, SW_FALSE, &sw_socket_tcp_connect, &sw_socket_tcp_send, &sw_socket_tcp_sendto, &sw_socket_tcp_recv, &sw_socket_tcp_recvfrom, &sw_socket_tcp_close));

	if ((self->m_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
   {
		sw_log(SW_LOG_ERROR, "sw_tcp_socket_init", "socket() failed: %d\n", sw_socket_error_code());
		return SW_E_FAIL;
	}

	return SW_OKAY;
}


sw_result
sw_tcp_socket_super_init_with_fd(
					sw_socket	self,
					int			fd)
{
   SW_TRY(sw_socket_init(self, SW_FALSE, &sw_socket_tcp_connect, &sw_socket_tcp_send, &sw_socket_tcp_sendto, &sw_socket_tcp_recv, &sw_socket_tcp_recvfrom, &sw_socket_tcp_close));

	self->m_fd = fd;

	return SW_OKAY;
}


sw_result
sw_udp_socket_super_init(
               sw_socket self)
{
   SW_TRY(sw_socket_init(self, SW_FALSE, &sw_socket_udp_connect, &sw_socket_udp_send, &sw_socket_udp_sendto, &sw_socket_udp_recv, &sw_socket_udp_recvfrom, &sw_socket_udp_close));

   if ((self->m_fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
   {
      sw_log(SW_LOG_ERROR, "sw_udp_socket_init", "socket() failed: %d\n", sw_socket_error_code());
      return SW_E_FAIL;
	}

	return SW_OKAY;
}


sw_result
sw_multicast_socket_super_init(
               sw_socket	self)
{
	sw_ulong	opt = 1;
   
   SW_TRY(sw_socket_init(self, SW_FALSE, &sw_socket_udp_connect, &sw_socket_udp_send, &sw_socket_udp_sendto, &sw_socket_udp_recv, &sw_socket_udp_recvfrom, &sw_socket_udp_close));

   if ((self->m_fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
   {
      sw_log(SW_LOG_ERROR, "sw_multicast_socket_init", "socket() failed: %d\n", sw_socket_error_code());
		return SW_E_FAIL;
   }

#if defined(__APPLE__) || defined(__VXWORKS__)
   if (setsockopt(self->m_fd, SOL_SOCKET, SO_REUSEPORT, (char*) &opt, sizeof(opt)) == -1)
#else
	if (setsockopt(self->m_fd, SOL_SOCKET, SO_REUSEADDR, (char*) &opt, sizeof(opt)) == -1)
#endif
	{
      sw_log(SW_LOG_ERROR, "sw_multicast_socket_init", "setsockopt(SO_REUSEADDR) failed: %d\n", sw_socket_error_code());
		return SW_E_FAIL;
	}

	return SW_OKAY;
}


sw_result
sw_socket_super_fina(
               sw_socket self)
{
	if (self != NULL)
	{
		sw_socket_close(self);
	}
   
   return SW_OKAY;
}


sw_ipv4_address HOWL_DLL
sw_socket_ipv4_address(
					sw_socket self)
{
	sw_ipv4_address address;

	sw_ipv4_address_init_from_saddr(&address, self->m_addr.sin_addr.s_addr);

	return address;
}


sw_port HOWL_DLL
sw_socket_port(
					sw_socket self)
{
	return ntohs(self->m_addr.sin_port);
}


sw_fd HOWL_DLL
sw_socket_fd(
					sw_socket self)
{
	return self->m_fd;
}


sw_result HOWL_DLL
sw_socket_bind(
               sw_socket			self,
					sw_ipv4_address	address,
               sw_port				port)
{
	sw_char		host[16];
   sw_optlen	len;
   
	sw_debug("sw_socket_bind() : fd = %d, addr = %s, port = %d\n", self->m_fd, sw_ipv4_address_name(address, host, 16), port);

   /*
      now bind
   */
	sw_memset(&self->m_addr, 0, sizeof(self->m_addr));
	self->m_addr.sin_family	=	AF_INET;
   
   /*
      if they haven't chosen a network interface, choose one for them
   */
	self->m_addr.sin_addr.s_addr	= 	sw_ipv4_address_saddr(address);
	self->m_addr.sin_port			=	htons(port);
   len									=	sizeof(self->m_addr);
   if (bind(self->m_fd, (struct sockaddr*) &self->m_addr, len) == -1)
   {
		sw_log(SW_LOG_ERROR, "sw_socket_bind", "bind() failed: %d\n", sw_socket_error_code());
		return sw_system_error();
   }
   
   /*
      now get our port
   */
   if (getsockname(self->m_fd, (struct sockaddr*) &self->m_addr, &len) == -1)
   {
      sw_log(SW_LOG_ERROR, "sw_socket_bind", "getsockname() failed: %d\n", sw_socket_error_code());
      return sw_system_error();
   }

   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_join_multicast_group(
               sw_socket			self,
					sw_ipv4_address	local_address,
					sw_ipv4_address	multicast_address,
               sw_ulong				ttl)
{
	struct in_addr				addr;
   struct ip_mreq				mreq;	
#if defined(__sun) || defined(__VXWORKS__)
	sw_uchar						real_ttl				= (sw_uchar) ttl;
#else
	sw_ulong						real_ttl				= ttl;
#endif

   /*
      initialize the group membership
   */
	sw_memset(&addr, 0, sizeof(addr));
	addr.s_addr = sw_ipv4_address_saddr(local_address);
   sw_memset(&mreq, 0, sizeof(mreq));

#if defined(__VXWORKS__)

	/*
		VxWorks doesn't like it if we try and use INADDR_ANY for use in
		setting up multicast sockets.

		So we please it.
	*/
	if (sw_ipv4_address_is_any(local_address))
	{
		sw_ipv4_address current_address;

		if (sw_ipv4_address_init_from_this_host(&current_address) != SW_OKAY)
		{
     		sw_log(SW_LOG_ERROR, "sw_socket_join_multicast_group", "sw_ipv4_address_init_from_this_host() failed\n");
      	return SW_E_INIT;
  		}

		mreq.imr_interface.s_addr	= sw_ipv4_address_saddr(current_address);
	}
	else
	{
		mreq.imr_interface = addr;
	}

#else

   mreq.imr_interface = addr;

#endif

	mreq.imr_multiaddr.s_addr = sw_ipv4_address_saddr(multicast_address);

	if (setsockopt(self->m_fd, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char*) &mreq, sizeof(mreq)) == -1)
	{
      sw_log(SW_LOG_ERROR, "sw_socket_join_multicast_group", "setsockopt(IP_ADD_MEMBERSHIP) failed: %d\n", sw_socket_error_code());
		return sw_system_error();
	}

	if (setsockopt(self->m_fd, IPPROTO_IP, IP_MULTICAST_IF, (char*) &addr, sizeof(addr)) == -1)
	{
		sw_log(SW_LOG_ERROR, "sw_socket_join_multicast_group", "setsockopt(IP_MULTICAST_IF) failed: %d\n", sw_socket_error_code());
		return sw_system_error();
	}

	if (setsockopt(self->m_fd, IPPROTO_IP, IP_TTL, (char*) &ttl, sizeof(ttl)) == -1)
	{
		sw_log(SW_LOG_ERROR, "sw_socket_join_multicast_group", "setsockopt(IP_TTL) failed: %d\n", sw_socket_error_code());
		return sw_system_error();
	}

	if (setsockopt(self->m_fd, IPPROTO_IP, IP_MULTICAST_TTL, (char*) &real_ttl, sizeof(real_ttl)) == -1)
	{
      sw_log(SW_LOG_ERROR, "sw_socket_join_multicast_group", "setsockopt(IP_MULTICAST_TTL) failed: %d\n", sw_socket_error_code());
      return sw_system_error();
	}

   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_listen(
               sw_socket	self,
               sw_ulong		qsize)
{
	sw_debug("sw_socket_listen() : fd = %d\n", self->m_fd);

   if (listen(self->m_fd, qsize) == -1)
   {
      sw_log(SW_LOG_ERROR, "sw_socket_listen", "listen() failed: %d\n", sw_socket_error_code());
      return sw_system_error();
   }
   
   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_connect(
               sw_socket			self,
               sw_ipv4_address	address,
               sw_port				port)
{
   return (*self->m_connect_func)(self, address, port);
}


sw_result HOWL_DLL
sw_socket_accept(
               sw_socket		self,
               sw_socket	*	socket)
{
   struct sockaddr_in	addr;
   sw_optlen				len;
   sw_fd						fd;
	int						nodelay = 1;
	struct linger			l;
   
	len = sizeof(addr);
   sw_memset(&addr, 0, sizeof(addr));
   if ((fd = accept(self->m_fd, (struct sockaddr*) &addr, &len)) < 0)
   {
		if (sw_socket_error_code() == EAGAIN)
		{
      	sw_log(SW_LOG_ERROR, "sw_socket_accept", "accept() failed: %d\n", sw_socket_error_code());
		}

      return sw_system_error();
   }
   
	if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char*) &nodelay, sizeof(nodelay)) == -1)
	{
		sw_log(SW_LOG_WARNING, "sw_socket_tcp_connect", "setsockopt(TCP_NODELAY) failed: %d\n", sw_socket_error_code());
	}
 
	l.l_onoff	= 0;
	l.l_linger	= 0;

	if (setsockopt(fd, SOL_SOCKET, SO_LINGER, (char*) &l, sizeof(l)) == -1)
	{
		sw_log(SW_LOG_WARNING, "sw_socket_tcp_connect", "setsockopt(SO_LINGER) failed: %d\n", sw_socket_error_code());
	}

   return sw_tcp_socket_init_with_fd(socket, fd);
}
                                    

sw_result HOWL_DLL
sw_socket_send(
               sw_socket	self,
               sw_octets	buffer,
               sw_len		len,
					sw_ulong	*	bytesWritten)
{
   return (*self->m_send_func)(self, buffer, len, bytesWritten);
}


sw_result HOWL_DLL
sw_socket_sendto(
					sw_socket			self,
					sw_octets			buffer,
					sw_len				len,
					sw_ulong			*	bytesWritten,
					sw_ipv4_address	to,
					sw_port				to_port)
{
	return (*self->m_sendto_func)(self, buffer, len, bytesWritten, to, to_port);
}


sw_result HOWL_DLL
sw_socket_recv(
               sw_socket	self,
               sw_octets	buffer,
               sw_len		max,
               sw_len	*	len)
{
   return (*self->m_recv_func)(self, buffer, max, len);
}


sw_result HOWL_DLL
sw_socket_recvfrom(
               sw_socket				self,
               sw_octets				buffer,
               sw_len					max,
               sw_len				*	len,
					sw_ipv4_address	*	from,
					sw_port				*	from_port,
					sw_ipv4_address	*	dest,
					sw_ulong				*	interface_index)
{
   return (*self->m_recvfrom_func)(self, buffer, max, len, from, from_port, dest, interface_index);
}


sw_result HOWL_DLL
sw_socket_set_blocking_mode(
				sw_socket	self,
				sw_bool		blocking_mode)
{
	sw_ulong i = (blocking_mode) ? 0 : 1;

#if defined(WIN32)
	if (ioctlsocket(self->m_fd, FIONBIO, &i) != 0)
	{
		sw_log(SW_LOG_ERROR, "sw_socket_set_blocking_mode", "ioctlsocket(FIONBIO) failed: %d\n", WSAGetLastError());
		return SW_E_INIT;
	}
#else
	if (ioctl(self->m_fd, FIONBIO, &i) != 0)
	{
		sw_log(SW_LOG_ERROR, "sw_socket_set_blocking_mode", "ioctl(FIONBIO) failed: %d\n", errno);
		return SW_E_INIT;
	}
#endif

	return SW_OKAY;
}


sw_long HOWL_DLL
sw_socket_last_error()
{
	return sw_socket_error_code();
}


sw_result HOWL_DLL
sw_socket_set_options(
               sw_socket				self,
               sw_socket_options		options)
{   
   if (options->m_debug.m_modified)
   {   
		sw_debug("sw_socket_set_options() : SO_DEBUG\n");

      if (setsockopt(self->m_fd, SOL_SOCKET, SO_DEBUG, (char*) &options->m_debug.m_val, sizeof(options->m_debug.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_DEBUG) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_debug.m_modified = SW_FALSE;
   }
   
#if !defined(__VXWORKS__)
   if (options->m_nodelay.m_modified)
   {      
		sw_debug("sw_socket_set_options() : TCP_NODELAY\n");

      if (setsockopt(self->m_fd, IPPROTO_TCP, TCP_NODELAY, (char*) &options->m_nodelay.m_val, sizeof(options->m_nodelay.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(TCP_NODELAY) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_nodelay.m_modified = SW_FALSE;
   }
#endif
   
   if (options->m_dontroute.m_modified)
   {      
		sw_debug("sw_socket_set_options() : SO_DONTROUTE\n");

      if (setsockopt(self->m_fd, SOL_SOCKET, SO_DONTROUTE, (char*) &options->m_dontroute.m_val, sizeof(options->m_dontroute.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_DONTROUTE) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_dontroute.m_modified = SW_FALSE;
   }
   
   if (options->m_keepalive.m_modified)
   {      
		sw_debug("sw_socket_set_options() : SO_KEEPALIVE\n");

      if (setsockopt(self->m_fd, SOL_SOCKET, SO_KEEPALIVE, (char*) &options->m_keepalive.m_val, sizeof(options->m_keepalive.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_KEEPALIVE) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_keepalive.m_modified = SW_FALSE;
   }

   if (options->m_linger.m_modified)
   {
		sw_debug("sw_socket_set_options() : SO_LINGER\n");

      if (setsockopt(self->m_fd, SOL_SOCKET, SO_LINGER, (char*) &options->m_linger.m_val, sizeof(options->m_linger.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_LINGER) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_linger.m_modified = SW_FALSE;
   }

   if (options->m_reuseaddr.m_modified)
   {      
		sw_debug("sw_socket_set_options() : SO_REUSEPORT\n");

#if defined(__APPLE__) || defined(__VXWORKS__)
		if (setsockopt(self->m_fd, SOL_SOCKET, SO_REUSEPORT, (char*) &options->m_reuseaddr.m_val, sizeof(options->m_reuseaddr.m_val)) == -1)
#else
		if (setsockopt(self->m_fd, SOL_SOCKET, SO_REUSEADDR, (char*) &options->m_reuseaddr.m_val, sizeof(options->m_reuseaddr.m_val)) == -1)
#endif
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_REUSEPORT) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_reuseaddr.m_modified = SW_FALSE;
   }

   if (options->m_sndbuf.m_modified)
   {      
		sw_debug("sw_socket_set_options() : SO_SNDBUF\n");

      if (setsockopt(self->m_fd, SOL_SOCKET, SO_SNDBUF, (char*) &options->m_sndbuf.m_val, sizeof(options->m_sndbuf.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_SNDBUF) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_sndbuf.m_modified = SW_FALSE;
   }

   if (options->m_rcvbuf.m_modified)
   {      
		sw_debug("sw_socket_set_options() : SO_RCVBUF\n");

      if (setsockopt(self->m_fd, SOL_SOCKET, SO_RCVBUF, (char*) &options->m_rcvbuf.m_val, sizeof(options->m_rcvbuf.m_val)) == -1)
      {
         sw_log(SW_LOG_ERROR, "sw_socket_set_options", "setsockopt(SO_RCVBUF) failed: %d\n", sw_socket_error_code());
         return sw_system_error();
      }
      
      options->m_rcvbuf.m_modified = SW_FALSE;
   }

   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_close(
               sw_socket	self)
{
   return (*self->m_close_func)(self);
}


static sw_result
sw_socket_init(
               sw_socket					self,
               sw_bool						connected,
               sw_socket_connect_func	connect_func,
               sw_socket_send_func		send_func,
					sw_socket_sendto_func	sendto_func,
               sw_socket_recv_func		recv_func,
					sw_socket_recvfrom_func	recvfrom_func,
               sw_socket_close_func		close_func)
{
	self->m_connected			=	connected;
   self->m_connect_func		=	connect_func;
   self->m_send_func			=	send_func;
	self->m_sendto_func		=	sendto_func;
   self->m_recv_func			=	recv_func;
	self->m_recvfrom_func	=	recvfrom_func;
   self->m_close_func		=	close_func;
	self->m_fd					=	-1;
   
   return SW_OKAY;
}


/*
	TCP implementation
*/
static sw_result
sw_socket_tcp_connect(
               sw_socket			self,
               sw_ipv4_address	address,
               sw_port				port)
{
	sw_char			host[16];
   sw_optlen		len;
	int				nodelay = 1;
	struct linger	l;
   
	sw_debug("entering sw_socket_tcp_connect() : host = %s, port = %d\n", sw_ipv4_address_name(address, host, 16), port);

   sw_memset(&self->m_dest_addr, 0, sizeof(self->m_dest_addr));
	self->m_dest_addr.sin_family		=	AF_INET;
	self->m_dest_addr.sin_addr.s_addr = 	sw_ipv4_address_saddr(address);
	self->m_dest_addr.sin_port			=	htons(port);

   if (connect(self->m_fd, (struct sockaddr*) &self->m_dest_addr, sizeof(self->m_dest_addr)) == -1)
	{
      sw_debug("sw_socket_tcp_connect: connect() failed: %d\n", sw_socket_error_code());
      return sw_system_error();
	}

	len = sizeof(self->m_addr);

	if (getsockname(self->m_fd, (struct sockaddr*) &self->m_addr, &len) == -1)
	{
      sw_log(SW_LOG_ERROR, "sw_socket_tcp_connect", "getsockname() failed: %d\n", sw_socket_error_code());
      return sw_system_error();
	}
   
   self->m_connected = SW_TRUE;

	if (setsockopt(self->m_fd, IPPROTO_TCP, TCP_NODELAY, (char*) &nodelay, sizeof(nodelay)) == -1)
	{
		sw_log(SW_LOG_WARNING, "sw_socket_tcp_connect", "setsockopt(TCP_NODELAY) failed: %d\n", sw_socket_error_code());
	}
 
	l.l_onoff	= 0;
	l.l_linger	= 0;

	if (setsockopt(self->m_fd, SOL_SOCKET, SO_LINGER, (char*) &l, sizeof(l)) == -1)
	{
		sw_log(SW_LOG_WARNING, "sw_socket_tcp_connect", "setsockopt(SO_LINGER) failed: %d\n", sw_socket_error_code());
	}
      
   return SW_OKAY;
}


static sw_result
sw_socket_tcp_send(
               sw_socket	self,
               sw_octets	bytes,
               sw_len		len,
					sw_ulong	*	bytesWritten)
{
   sw_long ret;
   
   sw_debug("sw_socket_tcp_send() entering: fd = %d\n", self->m_fd);

#if defined(WIN32)
	ret = send(self->m_fd, bytes, len, 0);
#else
	while (((ret = send(self->m_fd, bytes, len, 0)) == -1) && (sw_socket_error_code() == EINTR));
#endif

	if (ret > 0)
	{
		*bytesWritten = ret;
	}
   else if (ret == -1)
   {
#if defined(WIN32)
		if (WSAGetLastError() == WSAEWOULDBLOCK)
#else
		if (errno == EWOULDBLOCK)
#endif
		{
			return SW_E_WOULDBLOCK;
		}
		else
		{
      	sw_log(SW_LOG_ERROR, "sw_socket_tcp_send", "send() failed: %d on fd %d\n", sw_socket_error_code(), self->m_fd);
      	return sw_system_error();
   	}
	}
   
   sw_debug("sw_socket_tcp_send() sent: %d bytes on fd %d\n", ret, self->m_fd);
  
   return SW_OKAY;
}


static sw_result
sw_socket_tcp_sendto(
               sw_socket			self,
               sw_octets			buffer,
               sw_len				len,
					sw_ulong			*	bytesWritten,
					sw_ipv4_address	to,
					sw_port				to_port)
{
	return sw_socket_tcp_send(self, buffer, len, bytesWritten);
}


static sw_result
sw_socket_tcp_recv(
               sw_socket	self,
               sw_octets	buffer,
               sw_len		max,
               sw_len	*	len)
{
	sw_result	result;
   sw_long		ret;

   sw_debug("sw_socket_tcp_recv() entering: fd = %d, buffer = %x, max = %d)\n", self->m_fd, buffer, max);

	/*
	 * initialize
	 */
	result = SW_OKAY;

#if defined(WIN32)
	ret = recv(self->m_fd, buffer, max, 0);
#else
	while (((ret = recv(self->m_fd, buffer, max, 0)) == -1) && (sw_socket_error_code() == EINTR));
#endif

   if (ret == -1)
   {
		ret = 0;

		/*
		 * check for EWOULDBLOCK, because this socket might have been
		 * set up in non-blocking mode
		 */
#if defined(WIN32)
		if (sw_socket_error_code() != WSAEWOULDBLOCK)
#else
		if (sw_socket_error_code() != EWOULDBLOCK)
#endif
		{
			/*
			 * don't print out errors that are common
			 */
#if defined(WIN32)
			if (sw_socket_error_code() != WSAECONNRESET)
#else
			if (sw_socket_error_code() != ECONNRESET)
#endif
			{
				sw_log(SW_LOG_ERROR, "sw_socket_tcp_recv", "recv() failed: %d on fd %d\n", sw_socket_error_code(), self->m_fd);
			}

			result = SW_E_EOF;
		}
   }
   else if (ret == 0)
   {
      result = SW_E_EOF;
   }

	sw_debug("sw_socket_tcp_recv() received: %d bytes on fd %d\n", ret, self->m_fd);

	*len = ret;
   
	return result;
}


static sw_result
sw_socket_tcp_recvfrom(
               sw_socket				self,
               sw_octets				buffer,
               sw_len					max,
               sw_len				*	len,
					sw_ipv4_address	*	from,
					sw_port				*	from_port,
					sw_ipv4_address	*	dest,
					sw_ulong				*	interface_index)
{
	sw_ipv4_address_init_from_saddr(from, self->m_dest_addr.sin_addr.s_addr);
	*from_port = ntohs(self->m_dest_addr.sin_port);
	return sw_socket_tcp_recv(self, buffer, max, len);
}


static sw_result
sw_socket_tcp_close(
               sw_socket	self)
{
	sw_debug("sw_socket_tcp_close() : fd = %d\n", self->m_fd);

   shutdown(self->m_fd, 2);

#if defined(WIN32)
   closesocket(self->m_fd);
#else
   close(self->m_fd);
#endif
   
	self->m_connected = SW_FALSE;

   return SW_OKAY;
}


/*
   UDP/mcast implementation
*/
static sw_result
sw_socket_udp_connect(
               sw_socket			self,
               sw_ipv4_address	address,
               sw_port				port)
{      
   sw_memset(&self->m_dest_addr, 0, sizeof(self->m_dest_addr));
	self->m_dest_addr.sin_family		=	AF_INET;
	self->m_dest_addr.sin_addr.s_addr	=	sw_ipv4_address_saddr(address);
	self->m_dest_addr.sin_port			=	htons(port);
   self->m_connected						=	SW_TRUE;
   
   return SW_OKAY;
}


static sw_result
sw_socket_udp_send(
               sw_socket	self,
               sw_octets	buffer,
               sw_len		len,
					sw_ulong	*	bytesWritten)
{
	return sw_socket_udp_really_sendto(self, buffer, len, bytesWritten, (struct sockaddr*) &self->m_dest_addr, sizeof(self->m_dest_addr));
}


static sw_result
sw_socket_udp_sendto(		
               sw_socket			self,
               sw_octets			buffer,
               sw_len				len,
					sw_ulong			*	bytesWritten,
					sw_ipv4_address	to,
					sw_port				port)

{
	struct sockaddr_in addr;

	sw_memset(&addr, 0, sizeof(addr));
	addr.sin_family		= 	AF_INET;
	addr.sin_addr.s_addr	=	sw_ipv4_address_saddr(to);
	addr.sin_port			=	htons(port);
	
	return sw_socket_udp_really_sendto(self, buffer, len, bytesWritten, (struct sockaddr*) &addr, sizeof(addr));
}


static sw_result
sw_socket_udp_really_sendto(
               sw_socket				self,
               sw_octets				buffer,
               sw_len					len,
					sw_ulong				*	bytesWritten,
					struct sockaddr	*	to,
					sw_optlen				to_len)
{
   sw_long ret;

	struct sockaddr_in * addr = (struct sockaddr_in*) to;

   sw_debug("entering sw_socket_udp_really_sendto: dest %s %d\n", inet_ntoa(addr->sin_addr), ntohs(addr->sin_port));
   
#if defined(WIN32)
	ret = sendto(self->m_fd, buffer, len, 0, to, to_len);
#else
	while (((ret = sendto(self->m_fd, buffer, len, 0, to, to_len)) == -1) && (sw_socket_error_code() == EINTR));
#endif

	if (ret > 0)
	{
		*bytesWritten = ret;
	}
   else if (ret == -1)
   {
      sw_log(SW_LOG_ERROR, "sw_socket_udp_really_sendto", "sendto() failed: %d\n", sw_socket_error_code());
      return sw_system_error();
   }
  
   sw_debug("sw_socket_udp_really_sendto: sent %d bytes\n", ret);

   return SW_OKAY;
}


static sw_result
sw_socket_udp_recv(
               sw_socket		self,
               sw_octets		buffer,
               sw_len			max,
               sw_len		*	len)
{
   struct sockaddr_in	from;
	struct in_addr			dest;
	sw_ulong					interface_index;

	return sw_socket_udp_really_recvfrom(self, buffer, max, len, (struct sockaddr*) &from, sizeof(from), &dest, &interface_index);
}


static sw_result
sw_socket_udp_recvfrom(
					sw_socket				self,
					sw_octets				buffer,
					sw_len					max,
					sw_len				*	len,
					sw_ipv4_address	*	from,
					sw_port				*	from_port,
					sw_ipv4_address	*	dest,
					sw_ulong				*	interface_index)
{
	struct sockaddr_in	from_addr;
	struct in_addr			dest_addr;
	sw_result				result;

	result		= sw_socket_udp_really_recvfrom(self, buffer, max, len, (struct sockaddr*) &from_addr, sizeof(from_addr), &dest_addr, interface_index);
	sw_ipv4_address_init_from_saddr(from, from_addr.sin_addr.s_addr);
	*from_port	= ntohs(from_addr.sin_port);  

	if (dest)
	{
		sw_ipv4_address_init_from_saddr(dest, dest_addr.s_addr);
	}

	return result;
}


static sw_result
sw_socket_udp_really_recvfrom(
               sw_socket				self,
               sw_octets				buffer,
               sw_len					max,
               sw_len				*	len,
					struct sockaddr	*	from,
					sw_optlen				from_len,
					struct in_addr		*	dest,
					sw_ulong				*	interface_index)
{
	sw_long ret;

	sw_assert(len);
	sw_assert(from);
	sw_assert(dest);

#if defined(WIN32)
	ret = recvfrom(self->m_fd, buffer, max, 0, from, &from_len);
#else
	while (((ret = recvfrom(self->m_fd, buffer, max, 0, from, &from_len)) == -1) && (sw_socket_error_code() == EINTR));
#endif

   if (ret == -1)
   {
      sw_log(SW_LOG_ERROR, "sw_socket_udp_recv", "recvfrom() failed: %d\n", sw_socket_error_code());
      return sw_system_error();
   }

   sw_debug("sw_socket_udp_recv: received %d bytes\n", ret);

   *len = ret;
   
	return SW_OKAY;
}


static sw_result
sw_socket_udp_close(
               sw_socket	self)
{
#if defined(WIN32)
   closesocket(self->m_fd);
#else
   close(self->m_fd);
#endif

   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_options_init(
               sw_socket_options	*	options)
{
   SW_TRY(sw_malloc(*options, sw_socket_options, sizeof(struct _sw_socket_options)));
   
   sw_memset((*options), 0, sizeof(struct _sw_socket_options));
   
   return SW_OKAY;
}
                                    

sw_result HOWL_DLL
sw_socket_options_fina(
               sw_socket_options	options)
{
   sw_free(options);
   
   return SW_OKAY;
}
                                    
                                    
sw_result HOWL_DLL
sw_socket_options_set_debug(
               sw_socket_options	options,
               sw_bool				val)
{
   options->m_debug.m_val 			= val;
   options->m_debug.m_modified	=	SW_TRUE;
   return SW_OKAY;
}
                               
                               
sw_result HOWL_DLL
sw_socket_options_set_nodelay(
               sw_socket_options	options,
               sw_bool				val)
{
   options->m_nodelay.m_val 		= val;
   options->m_nodelay.m_modified	=	SW_TRUE;
   return SW_OKAY;
}
                                    
                                    
sw_result HOWL_DLL
sw_socket_options_set_dontroute(
               sw_socket_options	options,
               sw_bool				val)
{
   options->m_dontroute.m_val			=	val;
   options->m_dontroute.m_modified	=	SW_TRUE;
   return SW_OKAY;
}
                                    
                             
sw_result HOWL_DLL
sw_socket_options_set_keepalive(
               sw_socket_options	options,
               sw_bool				val)
{
   options->m_keepalive.m_val			=	val;
   options->m_keepalive.m_modified	=	SW_TRUE;
   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_options_set_linger(
               sw_socket_options	options,
               sw_bool				val,
               sw_ulong				linger)
{
   options->m_linger.m_val.l_onoff	=	val;
   options->m_linger.m_val.l_linger	=	(sw_ushort) linger;
   options->m_linger.m_modified		=	SW_TRUE;
   return SW_OKAY;
}
                                    
                                    
sw_result HOWL_DLL
sw_socket_options_set_reuseaddr(
               sw_socket_options	options,
               sw_bool				val)
{
   options->m_reuseaddr.m_val			=	val;
   options->m_reuseaddr.m_modified	=	SW_TRUE;
   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_options_set_sndbuf(
               sw_socket_options	options,
               sw_ulong				val)
{
   options->m_sndbuf.m_val			=	val;
   options->m_sndbuf.m_modified	=	SW_TRUE;
   return SW_OKAY;
}


sw_result HOWL_DLL
sw_socket_options_set_rcvbuf(
               sw_socket_options	options,
               sw_ulong				val)
{
   options->m_rcvbuf.m_val			=	val;
   options->m_rcvbuf.m_modified	=	SW_TRUE;
   return SW_OKAY;
}
